package editor;
import gui.CanvasDialogue;
import gui.RequestFocusListener;

import java.awt.Component;
import java.awt.Cursor;
import java.awt.Dimension;
import java.awt.GridLayout;
import java.awt.Image;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Arrays;

import javax.swing.BorderFactory;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JPasswordField;
import javax.swing.border.Border;
import javax.swing.border.CompoundBorder;
import javax.swing.border.EtchedBorder;
import javax.swing.border.TitledBorder;

import admin.VentanaAdminUsuarioSel;

import utils.DataUsers;
import utils.Global;

/**
 * Builds a login dialogue with a hidden button at the top left corner. The central section is 
 * made up of a list of buttons each representative of a user. If the number of users is greater
 * than a certain maximum, the remaining user buttons will be organised into further "pages". Navigation
 * through the different pages is provided through two buttons in the botton section: "back" and
 * "more people".  
 * @author hqcasanova
 * 
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this file,
 * You can obtain one at http://mozilla.org/MPL/2.0/.
 */
public class Login extends CanvasDialogue implements ActionListener {
	//Central panel for the user list
	private JPanel usrListPanel;
	
	//Interactive elements
	private final JButton adminButton, moreButton, backButton;
	private JButton[] usrButtons;
		
	//Others
	private Editor editor;			//object representing editor's main window
	private final ImageIcon passwordIcon;	//icon used in admin's password dialogue
	private int iDisplayedUsers,			//total number of users displayed up to the present page
	            iPage,						//current page number
	            iUsrWidth,					//width of the user button
	            iUsrHeight;					//height of the user button
	
	/**
	 * Customised constructor. Builds the page navigation and administrative buttons. The list
	 * of users is populated and the first page displayed.
	 * @param sTitle Title of the login window
	 * @param iAdminButtonWidth Width of hidden admin button
	 * @param iAdminButtonHeight Height of hidden admin button 
	 * @param iUsrWidth Width of user button
	 * @param iUsrHeight Height of user button
	 * @param editor Object representing editor's main window
	 */
	public Login (String sTitle, int iAdminButtonWidth, int iAdminButtonHeight, 
			                int iUsrWidth, int iUsrHeight, Editor editor) {
		//Non-resizable empty dialogue without background. By default, x button closes the whole app
		super (sTitle, editor.getWindow(), null, null, null, true, JDialog.EXIT_ON_CLOSE, Global.FAVICON_ED, false);
			
		//Initialises user button dimensions.
		this.iUsrWidth = iUsrWidth;
		this.iUsrHeight = iUsrHeight;
		
		//If there is an editor main window, the login window's close button closes just the login window 
		this.editor = editor;
				
		//Sets up the icon for the password dialogue
		passwordIcon = new ImageIcon (Global.IMG_FOLDER +Global.SEPARADOR_SYS+ Global.ADMINLOCK+ Global.PICT_EXT_PUNTO);
		passwordIcon.setImage (passwordIcon.getImage().getScaledInstance (Global.ICON_SIZE, Global.ICON_SIZE,
				            Image.SCALE_AREA_AVERAGING));
		
		//Sets up all non-user buttons
		adminButton = new JButton ();
		adminButton.setPreferredSize (new Dimension (iAdminButtonWidth, iAdminButtonHeight));
		adminButton.addActionListener (this);
		moreButton = new JButton (Global.LANG.getString ("loginBotonMas"));
		moreButton.addActionListener (this);
		backButton = new JButton (Global.LANG.getString ("loginBotonMenos"));
		backButton.addActionListener (this);
		backButton.setVisible (false);
		
		//Hides the admin button
		adminButton.setOpaque (false);
		adminButton.setContentAreaFilled (false); 
		adminButton.setBorderPainted (false);
		adminButton.setFocusable (false);
		
		//Adds all non-user buttons to the dialogue's top and bottom sections
		Component[] topLeft = new Component[1];
		topLeft[0] = adminButton;
		Component[] bottomLeft = new Component[1];
		bottomLeft[0] = backButton;
		Component[] bottomRight = new Component[2];
		bottomRight[0] = new JLabel();
		bottomRight[1] = moreButton;
		
		//Sets up the panel containing the user list 
		usrListPanel = new JPanel (new GridLayout (0, 3, 10, 10));
		usrListPanel.setBorder (BorderFactory.createTitledBorder 
				                (Global.LANG.getString ("loginGrupoTitulo")));
		configUserButtons();
				
		//Resets the centre panel to the the first page of users
		reset();
				
		//Adds all the components to the window
		populate (topLeft, usrListPanel, bottomLeft, bottomRight);		
    }
	
	/**
	 * Goes back to the first page of the user list
	 */
	private void reset () {
		iDisplayedUsers = 0;
		iPage = 0;
		updateUsrList();
	}
	
	/**
	 * Retrieves all user buttons and sets them up as event sources for this dialogue.
	 */
	private void configUserButtons () {
		int iSizeWithoutNulls; //size of the button array without nulls.
		
		//Retrieves the set of user buttons to be displayed.
		usrButtons = DataUsers.getUsersButtons (iUsrWidth, iUsrHeight);
		
		//Finds out the real size of the user button (some users might have been set as hidden)
		iSizeWithoutNulls = 0; 
		while ((iSizeWithoutNulls < usrButtons.length) && (usrButtons[iSizeWithoutNulls] != null)) {
			  usrButtons[iSizeWithoutNulls].setCursor (Cursor.getPredefinedCursor (Cursor.HAND_CURSOR));
			  usrButtons[iSizeWithoutNulls].addActionListener (this);
			  iSizeWithoutNulls++;
		}
		
		//Strips all nulled positions out of the user button array
		usrButtons = Arrays.copyOf (usrButtons, iSizeWithoutNulls);
		
		//If all users fit in one page, hide the "more" button
		if (usrButtons.length <= Global.NMAXPAGUSER)
		   moreButton.setVisible (false);
		else moreButton.setVisible (true);
	}
	
	/**
	 * Populates the panel containing the user list with the corresponding buttons.
	 */
	private void updateUsrList () {
		//Sets the mouse pointer to busy
		busy (true);
		
		//Removes all user buttons currently being displayed
		usrListPanel.removeAll();
		
		//Adds all user buttons to the current page until the maximum is reached
	    do {
	       usrListPanel.add (usrButtons[iDisplayedUsers]);
		   iDisplayedUsers++;
		} while ((iDisplayedUsers % Global.NMAXPAGUSER != 0) && (iDisplayedUsers < usrButtons.length));
	      
	    //Resets the mouse pointer and refreshes the window
	    busy (false);
	    refresh();
	}
	
	/**
	 * Manages the events coming from all buttons
	 */
	public void actionPerformed (ActionEvent e) {
		String sNameClickedUsr;
		int iIdUsr;
		 
		//Hidden admin button
		if (e.getSource() == adminButton) {
			
   		   //Title for the password field
		   JLabel pwdFieldTitle = new JLabel (Global.LANG.getString ("loginAdminTexto"));
		   			
		   //Sets up the password field with top priority for focus.	
		   JPasswordField pwdField = new JPasswordField();
		   pwdField.addAncestorListener (new RequestFocusListener());
		   	   
		   //Displays the password dialogue and retrieves the exit code
		   int result = JOptionPane.showOptionDialog (getWindow(), new Object[] {pwdFieldTitle, pwdField}, 
				   					Global.LANG.getString ("loginAdminTitulo"), JOptionPane.OK_CANCEL_OPTION, 
				   					JOptionPane.INFORMATION_MESSAGE, passwordIcon, 
				   					new String[] {Global.LANG.getString ("botonAceptar"), 
			                                      Global.LANG.getString ("botonCancelar")}, 
				   					Global.LANG.getString("botonAceptar"));
			                               
		   
		   //If ok button pressed, check password and refresh user list
		   if (result == 0) {
			  if (DataUsers.authAdmin (new String (pwdField.getPassword()))) {
				  
				 //Sets up the main administration window, displays it and waits for user interaction
                 VentanaAdminUsuarioSel interfazAdmin = new VentanaAdminUsuarioSel (1);
                 interfazAdmin.setLocationRelativeTo (editor.getWindow());
                 interfazAdmin.setVisible (true);
                 
                 //Refreshes user list
                 configUserButtons();
                 reset();
                 
                 //If the current user was modified, refreshes the editor's main window too
                 if (interfazAdmin.isUsrCambiado (editor.getUsrId()))
                    editor.loadUsr (editor.getUsrId(), editor.getUsrName());
                 
                 //If the document currently open has been deleted from the database... 
                 if (editor.isDocOpen() && !DataUsers.existDocumentTitle (Integer.toString (editor.getUsrId()), editor.getDocName())) {
                	
                	//Displays a message telling the user about it
                	JOptionPane.showMessageDialog (super.getWindow(),
                             Global.LANG.getString ("adminErrorDocTexto"), 
                             Global.LANG.getString ("adminErrorDocTitulo"), 
                             JOptionPane.INFORMATION_MESSAGE); 
                	
                	//Displays the new document dialogue
                	SelectDoc eligeNuevo = new SelectDoc (Global.LANG.getString ("nuevoTitulo"),
                            20, 80, editor, null);
                	eligeNuevo.display(getWindow());                	
                 }
                                 
              //The password is wrong => displays message for user   
			  } else JOptionPane.showMessageDialog (super.getWindow(),
                          Global.LANG.getString ("adminErrorTexto"), 
                          Global.LANG.getString ("adminErrorTitulo"), 
                          JOptionPane.ERROR_MESSAGE);			  
		   }
		   
		//"More" button
		} else if (e.getSource() == moreButton) {
			      backButton.setVisible (true);
			      iPage++;
			      
			      //Already on the last page? => hide "more" button
			      if ((iPage + 1) * Global.NMAXPAGUSER >= usrButtons.length)
			    	 moreButton.setVisible (false); 
			     			      
			      //Refreshes the window with the user buttons 
			      updateUsrList();
	    	      
	    //"Back" button
		} else if (e.getSource() == backButton) {
			      iPage--;
			      iDisplayedUsers  = iPage * Global.NMAXPAGUSER;
			      
			      //More users left after the ones on this page? => display "more" button.
			      if ((iPage * Global.NMAXPAGUSER) < usrButtons.length)
			    	 moreButton.setVisible (true);
			      
			      //No need for "back" button on the first page
			      if (iPage == 0)
			    	 backButton.setVisible (false);
			      
			      //Refreshes the window with the user buttons 
			      updateUsrList();
			      
	    //A user button has been pressed
		} else {
			   //Find out name and ID of selected user
			   iIdUsr = Integer.valueOf (e.getActionCommand());
			   sNameClickedUsr = ((JButton) e.getSource()).getText();			   
			   		   
			   //Closes the login dialogue and refreshes the editor's main window
			   close();
			   editor.loadUsr (iIdUsr, sNameClickedUsr);		   
		}
	}
	
	/**
	 * Resets the contents of the dialogue every time is displayed for the first time
	 */
	public void display () {
		reset();
		display (editor.getWindow());		
	}
}
