package gui;
import java.awt.Graphics2D;
import java.awt.RenderingHints;
import java.awt.Transparency;
import java.awt.Window;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.FilenameFilter;
import java.io.IOException;

import javax.imageio.ImageIO;
import javax.swing.BorderFactory;
import javax.swing.Icon;
import javax.swing.ImageIcon;
import javax.swing.JFrame;
import javax.swing.JProgressBar;

import utils.Global;

/**
 * Provides methods for modifying a series of image files
 * @author hqcasanova
 * 
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this file,
 * You can obtain one at http://mozilla.org/MPL/2.0/.
 */
public class ImageArray {
	private BufferedImage[] images; //Images loaded in memory
	private String[] files;			//File names
	private String sExtension;		//Common image file extensions
	private int iArraySize;			//Number of images
	
	/**
	 * Customised constructor. Loads a series of image files into buffered memory.
	 * @param files File name array
	 * @param sPath Common path to image files
	 * @param sExtension Common image file extension
	 */
	public ImageArray (String[] files, String sPath, String sExtension) {
		int i = 0;
		File imgFile;
		
		iArraySize = files.length;
		this.files = new String[iArraySize];
		this.sExtension = sExtension;		
		try {
			images = new BufferedImage[iArraySize];
			for (i = 0; i < files.length; i++) {
				this.files[i] = files[i]+ "." +sExtension;
				imgFile = new File (sPath +this.files[i]);
				images[i] = ImageIO.read (imgFile);
				imgFile = null;
			}
		} catch (Exception e) {
			    System.out.println ("Ha ocurrido un problema al leer la imagen " +sPath+this.files[i]+ ": " +e);
			    try {
			    	imgFile = new File (Global.FILE_ERROR_IMAGE);
					images[i] = ImageIO.read (new File (Global.FILE_ERROR_IMAGE));
					imgFile = null;
				} catch (IOException eIO) {
					    System.out.println ("El fichero " +Global.FILE_ERROR_IMAGE+ " no existe.");
					    eIO.printStackTrace();
				}	    
		}
	}
	
	/**
	 * Customised constructor for a single image file.
	 * @param sFile Image file name
	 * @param sPath Image file path
	 * @param sExtension Image file extension
	 * @param sErrorPath Path to the fallback image file in case of error
	 */
	public ImageArray (String sFile, String sPath, String sExtension) {
		File imgFile;		
		
		iArraySize = 1;
		this.files = new String[1];
		this.files[0] = sFile+ "." +sExtension;
		this.sExtension = sExtension;
		images = new BufferedImage[1];
		try {
			imgFile = new File (sPath +sFile+ "." +sExtension);
			images[0] = ImageIO.read (imgFile);
			imgFile = null;
		} catch (Exception e) {
			    System.out.println ("Ha ocurrido un problema al leer la imagen " +sPath+sFile+"."+sExtension+ ": " +e);
			    try {
			    	imgFile = new File (Global.FILE_ERROR_IMAGE);
					images[0] = ImageIO.read (new File (Global.FILE_ERROR_IMAGE));
					imgFile = null;
				} catch (IOException eIO) {
					    System.out.println ("El fichero " +Global.FILE_ERROR_IMAGE+ " no existe.");
					    eIO.printStackTrace();
				}	    
		}
	}
	
	/**
	 * Changes the size of the images. To that end, it redraws the original image but with the new
	 * dimensions. If one of the dimensions provided is negative, this is re-calculated from the 
	 * other dimension while keeping the aspect ratio.
	 * @param iFinalWidth Scaled image's width. If negative, a new width is worked out.
	 * @param iFinalHeight Scaled image's height. If negative, a new height is worked out.
	 * @return Total height of all scaled images.
	 * Note: modified version of code by Chris Campbell 
	 * http://today.java.net/pub/a/today/2007/04/03/perils-of-image-getscaledinstance.html
	 */
	public int resize (int iFinalWidth, int iFinalHeight) {
        BufferedImage img;
        int iWidth, iHeight, type, i,
            iOrigHeight, iOrigWidth,
            iTotalHeight;
                        
        iOrigWidth = iFinalWidth;
        iOrigHeight = iFinalHeight;
        iTotalHeight = 0;
        
        //For every image...
        for (i = 0; i < iArraySize; i++) {
			   	
        	//Sets the appropriate type of transparency
        	type = (images[i].getTransparency() == Transparency.OPAQUE) ? BufferedImage.TYPE_INT_RGB : BufferedImage.TYPE_INT_ARGB;
           
        	//Finds out the image's dimensions
			iWidth = images[i].getWidth();
            iHeight = images[i].getHeight();
             
            //If any dimension is negative, work it out taking aspect ratio into account
            if (iFinalWidth < 0) 
               iFinalWidth = (int) (iFinalHeight * ((double) iWidth / (double) iHeight));
            if (iFinalHeight < 0)
               iFinalHeight = (int) (iFinalWidth * ((double) iHeight / (double) iWidth));	
                        
            //If the image is sized up...
            if (iWidth < iFinalWidth) 
               iWidth =	iFinalWidth;
            if (iHeight < iFinalHeight) 
               iHeight = iFinalHeight;
                            
            //When sizing down, repeat scaling with every fraction of dimension
            do {
               //Half dimensions
               if (iWidth > iFinalWidth) {
                  iWidth /= 2;
                  if (iWidth < iFinalWidth)
                     iWidth = iFinalWidth;
               }
               if (iHeight > iFinalHeight) {
                  iHeight /= 2;
                  if (iHeight < iFinalHeight) 
                     iHeight = iFinalHeight;
               }
               
               //Scale the image
               img = new BufferedImage (iWidth, iHeight, type);
               Graphics2D g2 = img.createGraphics();
               g2.setRenderingHint (RenderingHints.KEY_INTERPOLATION, RenderingHints.VALUE_INTERPOLATION_BILINEAR);
               g2.drawImage (images[i], 0, 0, iWidth, iHeight, null);
               g2.dispose();
               images[i] = img;
            } while ((iWidth != iFinalWidth) || (iHeight != iFinalHeight));
            
            //Add up to the total height of all consecutive images
            iTotalHeight = iFinalHeight + iTotalHeight;
            
            //Recover the originally desired width and height
            iFinalWidth = iOrigWidth;
			iFinalHeight = iOrigHeight;
        }   
        
        return iTotalHeight;
	}

	
	/**
	 * Converts all images to icon objects
	 * @return Icon object array.
	 */
	public Icon[] toIcons () {
		Icon[] iconos;
		int i;
		
		iconos = new Icon[iArraySize];
		for (i = 0; i < iArraySize; i++)
			iconos[i] = new ImageIcon (images[i]);
	    return iconos;
	}
	
	/**
	 * Converts only the first image to an icon object. Useful when using the class for
	 * a single image.
	 * @return Icon object.
	 */
	public Icon toIcon () {
		return toIcons()[0]; 	
	}
	
	/**
	 * Saves all the buffered images as files at a given path.
	 * @param sNewPath New location for the buffered images
	 */
	public void toFile (String sNewPath) {
		int i;
		for (i = 0; i < iArraySize; i++)
			try {
				ImageIO.write (images[i], sExtension, new File (sNewPath +Global.SEPARADOR_SYS+ files[i]));
			} catch (IOException eIO) {
				    System.out.println ("Error al crear fichero de imagen: " +eIO);
					eIO.printStackTrace();
			}
	}
	
	/**
	 * Resizes all images from a given source path and saves them to an also given destination path 
	 * A progress bar is displayed for visual feedback.
	 * @param sSourceImg Source path of the images to be scaled
	 * @param sDestinationImg Destination path for the scaled version of the images
	 * @param sExtension Image file extension
	 * @param iWidth New image width
	 * @param iHeight New image height
	 * @param sTitle Title of the progress bar window.
	 * @param owner Owner the progress bar window should center relative to.
	 */
	public static void resizeDir (final String sSourceImg, final String sDestinationImg, 
			                      final String sExtension, final int iWidth, final int iHeight,
			                      String sTitle, Window owner) {
		JProgressBar progress;	//progress bar object
		String[] files;			//image file names
		CanvasDialogue progressDiag;	//dialogue window containing the progress bar 
		int i;					
		
		//Sets up the progress bar dialogue and displays it
		progress = new JProgressBar();
		progress.setStringPainted (true);
		progress.setIndeterminate (true);
		progressDiag = new CanvasDialogue (sTitle, owner, null, null, null, false, 
				                    JFrame.DO_NOTHING_ON_CLOSE, Global.FAVICON_ED, false);
		progressDiag.getWindow().setModal (false);
		progressDiag.getCanvas().setBorder (BorderFactory.createEmptyBorder (10, 30, 10, 30));
		progressDiag.populate (progress);
		progressDiag.display (owner);
		progressDiag.busy (true);
		
		//Retrieves the names of all image files through a filter.
		files = new File (sSourceImg).list (new FilenameFilter() {                  
                    	public boolean accept (File directorio, String nombreFich)  {
                    		return nombreFich.endsWith ("." + sExtension);
                    	}
				   });
		
		//Initialises the progress bar
		progress.setMaximum (files.length);
		progress.setIndeterminate (false);
				
		//Loads each image, scales it and saves it. The saving is threaded.
		for (i = 0; i < files.length; i++) { 
			files[i] = files[i].substring(0, files[i].lastIndexOf('.'));
			final ImageArray imagen = new ImageArray (files[i], sSourceImg + Global.SEPARADOR_SYS, sExtension);
			imagen.resize (iWidth, iHeight);
			progress.setValue (i + 1);
			progress.paintImmediately (progress.getVisibleRect());
			new Thread() {
				public void run() {
					imagen.toFile (sDestinationImg);					
				}
			}.start ();			
		}	
		
		//Closes the progress window
		progressDiag.close();
	}
}
